package Level3;

sub check_return{
	###SpecialCase return 'Level3 SpecialCase uncovered';
	###Meditation return 'Level3 Peace uncovered';
	###LogSD return 'Level3 LogSD uncovered';
	return 'No Level3 uncovering occured';
}

1;